from typing import Any
# from .IUsersTable import IUsersTable


class IDatabase:
    users: Any

    def connect(self): ...
    def execute(self, *args, **kwargs) -> Any: ...
    def commit(self): ...
    def close(self): ...




import sqlite3
import json
from roombot.interfaces.IDatabase import IDatabase
from .json_users import JsonUsersTable
from typing import Any


class JsonDatabase(IDatabase):
    conn: dict
    users: JsonUsersTable
    filename: str

    def __init__(self, filename: str):
        self.filename = filename
        self.users = JsonUsersTable(self)

    def connect(self):
        self.conn = json.load(open(self.filename, "r"))

    def execute(self, request, arguments: list = None) -> Any:
        pass

    def commit(self):
        json.dump(self.conn, open(self.filename, "w"))

    def close(self):
        self.commit()



import sqlite3
import json
from roombot.interfaces.IDatabase import IDatabase
from .sqlite_users import Sqlite3Users
from typing import Any


def dict_factory(cursor, row):
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d


class Sqlite3Database(IDatabase):
    conn: sqlite3.Connection = None
    cursor: sqlite3.Cursor = None
    users: Sqlite3Users
    filename: str

    def __init__(self, filename: str):
        self.users = Sqlite3Users(self)
        self.filename = filename

    def connect(self):
        self.conn = sqlite3.connect(self.filename)
        self.conn.row_factory = dict_factory
        self.cursor = self.conn.cursor()

    def execute(self, request, arguments: list = None) -> Any:
        if self.cursor:
            self.cursor.execute(request, arguments)
            return self.cursor.fetchall()

    def commit(self):
        self.conn.commit()

    def close(self):
        self.cursor.close()
        self.conn.close()



from dataclasses import dataclass


@dataclass
class User:
    telegram_id: int
    firstname: str
    lastname: str
    permissions: str
    room: str
    id: int = None

    def as_list(self):
        if self.id:
            return [self.telegram_id, self.firstname, self.lastname, self.permissions, self.room, self.id]
        else:
            return [self.telegram_id, self.firstname, self.lastname, self.permissions, self.room]

    def as_dict(self):
        return {"telegram_id": self.telegram_id,
                "firstname": self.firstname,
                "lastname": self.lastname,
                "permissions": self.permissions,
                "room": self.room,
                "id": self.id
                }



from typing import List
from roombot.types.room import Room
from roombot.types.handler import Handler
from roombot.types.handler import HandlersTypes
from roombot.exception.room_container import IncorrectHandlerType


class RoomsContainer:
    rooms: List[Room]

    def __init__(self):
        self.rooms = []

    def add_room(self, name: str, room_type: int, content_type=None, permissions=None, **kwargs):
        if content_type is None:
            content_type = []
        if permissions is None:
            permissions = []

        def decorator(parameters):
            if isinstance(parameters, dict):
                past_room_type = parameters.get("past_room_type")
                func = parameters.get("func")
            else:
                past_room_type = None
                func = parameters
            if past_room_type is not None:
                if not HandlersTypes.are_compatibility(past_room_type, room_type):
                    raise Exception("Un compatibility rooms. ")
            if not HandlersTypes.in_types(room_type):
                raise IncorrectHandlerType("Incorrect handler type")
            room_handler = Handler(room_type, content_type, func)
            room = Room(name, room_handler, kwargs, permissions)
            self.rooms.append(room)
            return {"func": func, "past_room_type": room.handler.handler_type}
        return decorator




from roombot.types.dataclasses import User
from roombot.types.room import Room, RoomsStack
from roombot.interfaces.IUsersTable import IUsersTable


class ActionsWithUser:
    def __init__(self, users: IUsersTable, internal_data, rooms: RoomsStack):
        self.users = users
        self.internal_data = internal_data
        self.rooms = rooms

    async def go_to_room(self, user: User or int, room_name: str) -> int:
        if isinstance(user, int):
            user = await self.users.get_user_by_telegram_id(user)
            user: User

        if self.internal_data.users_migrations.get(user.telegram_id):
            return 2  # can't make recourse traveling

        for room in self.rooms.get(room_name):
            if self.user_can_go_to_room(user, room):
                self.internal_data.users_migrations[user.telegram_id] = room_name
                return 1  # can go
        return 0  # can't go

    async def set_permissions(self, user: User or int, permissions: str):
        if isinstance(user, User):
            await self.users.set_user_permission_by_telegram_id(user.telegram_id, permissions)
        elif isinstance(user, int):
            await self.users.set_user_permission_by_telegram_id(user, permissions)

    async def get_permissions(self, user: User or int):
        if isinstance(user, User):
            return (await self.users.get_user_by_telegram_id(user.telegram_id)).permissions
        elif isinstance(user, int):
            return (await self.users.get_user_by_telegram_id(user)).permissions

    def user_can_go_to_room(self, user: User, room: Room):
        myif = (user.permissions in room.permissions or ((user.permissions == self.internal_data.default_permissions or not len(user.permissions)) and not len(room.permissions))) or not len(room.permissions)
        if myif:
            return True
        return False




import aiogram

from roombot.manager.manager import RoomsManager
from roombot.interfaces.IBot import IBot
from aiogram.dispatcher.middlewares import BaseMiddleware


class MessageTypes:
    TEXT = "text"
    AUDIO = "audio"
    DOCUMENT = "document"
    GAME = "game"
    PHOTO = "photo"
    STICKER = "sticker"
    VIDEO = "video"
    VIDEO_NOTE = "video_note"
    VOICE = "voice"
    CONTACT = "contact"
    LOCATION = "location"
    VENUE = "venue"
    NEW_CHAT_MEMBERS = "new_chat_members"
    LEFT_CHAT_MEMBER = "left_chat_member"
    INVOICE = "invoice"
    SUCCESSFUL_PAYMENT = "successful_payment"
    CONNECTED_WEBSITE = "connected_website"
    MIGRATE_TO_CHAT_ID = "migrate_to_chat_id"
    MIGRATE_FROM_CHAT_ID = "migrate_from_chat_id"
    UNKNOWN = "unknown"
    ANY = "any"


class AiogramMiddleWare(BaseMiddleware):
    bot: aiogram.Bot
    dispatcher: aiogram.Dispatcher

    def __init__(self, rooms_manager: RoomsManager):
        self.rooms_manager = rooms_manager
        super().__init__()

    async def on_post_process_message(self,
                                      message: aiogram.types.Message,
                                      results: tuple,
                                      data: dict
                                      ):
        await self.rooms_manager.process_message(message)

    async def on_post_process_callback_query(self,
                                             callback: aiogram.types.CallbackQuery,
                                             results: tuple,
                                             data: dict
                                             ):
        await self.rooms_manager.process_callback(callback)




from typing import Any
from .IDatabase import IDatabase
from roombot.types.dataclasses import User


class IUsersTable:
    # VAR
    database: IDatabase
    # GET
    def __init__(self, database: IDatabase): self.database = database
    async def get_user_by_id(self, user_id: int) -> User or None: ...
    async def get_user_by_telegram_id(self, telegram_id: int) -> User or None: ...
    # ADD
    async def add_user(self, user: User) -> User: ...
    # SET
    async def edit_user_data(self, user: User) -> Any: ...
    async def set_user_room_by_id(self, user_id: int, room: str) -> bool: ...
    async def set_user_room_by_telegram_id(self, telegram_id: int, room: str) -> bool: ...
    async def set_user_permission_by_telegram_id(self, telegram_id: int, permission: str) -> bool: ...




from roombot.interfaces.IUsersTable import IUsersTable
from roombot.interfaces.IDatabase import IDatabase
from roombot.types.dataclasses import User


class JsonUsersTable(IUsersTable):
    database: IDatabase

    async def add_user(self, user: User) -> User:
        uid = len(self.database.conn.get("users"))
        user.id = uid
        self.database.conn["users"].append(user.as_dict())
        self.database.commit()
        return user

    async def get_user_by_id(self, user_id: int) -> (User or None):
        for usr in self.database.conn.get("users"):
            if usr.get("id") == user_id:
                return User(**usr)
        return None

    async def get_user_by_telegram_id(self, telegram_id: int) -> (User or None):
        for usr in self.database.conn.get("users"):
            if usr.get("telegram_id") == telegram_id:
                # print(usr)
                return User(**usr)
        return None

    async def set_user_room_by_id(self, user_id: int, room: str) -> bool:
        num = 0
        for usr in self.database.conn.get("users"):
            if usr.get("id") == user_id:
                self.database.conn["users"][num]["room"] = room
                self.database.commit()
                break
            num += 1
        return True

    async def set_user_room_by_telegram_id(self, telegram_id: int, room: str) -> bool:
        num = 0
        for usr in self.database.conn.get("users"):
            if usr.get("telegram_id") == telegram_id:
                self.database.conn["users"][num]["room"] = room
                self.database.commit()
                break
            num += 1
        return True

    async def set_user_permission_by_telegram_id(self, telegram_id: int, permission: str) -> bool:
        num = 0
        for usr in self.database.conn.get("users"):
            if usr.get("telegram_id") == telegram_id:
                self.database.conn["users"][num]["permissions"] = permission
                self.database.commit()
                break
            num += 1
        return True




from dataclasses import dataclass
from tgusers.dataclasses.args_container import ArgumentsBox
from typing import List
from tgusers.dataclasses.rooms import Argument


class ArgumentsContainer:
    def __init__(self):
        self.arguments_list: List[ArgumentsBox]
        self.arguments_list = []


    def add_arguments_to_room(self, room_name: str, arguments: dict):
        """
        :param room_name:
        :param arguments:
            Dict of {
                        annotation: value,
                        annotation1: value1,
                        annotation2: value2,
                    }
        :return:
        """
        if not isinstance(arguments, dict):
            raise Exception("Arguments must be an <dict>")
        arguments_list = []
        for argument in arguments:
            one_argument = Argument(value=arguments.get(argument), annotation=argument)
            arguments_list.append(one_argument)
        argument_pack = ArgumentsBox(room_name, arguments_list)
        self.arguments_list.append(argument_pack)



import inspect


async def run_func_as_async(function, *func_args, **func_kwargs):
    if inspect.iscoroutinefunction(function):
        return await function(*func_args, **func_kwargs)
    else:
        return function(*func_args, **func_kwargs)


def get_kwargs_by_annotations(function, arguments):
    params = list(inspect.signature(function).parameters.items())
    func_args = {}
    num = 0
    for param in params:
        if num >= len(arguments):
            break
        _, parameter = param
        func_args[parameter.name] = arguments.get(parameter.annotation)
        if not func_args[parameter.name]:
            pass
        num += 1
    return func_args


def get_only_the_required_arguments(function, arguments):
    params = list(inspect.signature(function).parameters.items())
    func_args = {}
    for param in params:
        _, parameter = param
        if parameter.name in arguments:
            func_args[parameter.name] = arguments.get(parameter.name)
    return func_args




from roombot.interfaces.IUsersTable import IUsersTable
from roombot.types.dataclasses import User


class Sqlite3Users(IUsersTable):
    async def add_user(self, user: User) -> User:
        sql1 = """
        INSERT INTO users (telegram_id, firstname, lastname, permissions, room)
        VALUES (?, ?, ?, ?, ?);
        """
        sql2 = """
        SELECT id
        FROM users
        WHERE telegram_id = ?;"""
        uid = self.database.execute(sql2, [user.telegram_id, ])
        if uid:
            user.id = uid[0].get("id")
            return user
        self.database.execute(sql1, user.as_list())
        self.database.commit()
        uid = self.database.execute(sql2, [user.telegram_id, ])
        user.id = uid[0].get("id")
        return user

    async def get_user_by_id(self, user_id: int) -> (User or None):
        sql = """
                SELECT * 
                FROM users
                WHERE id = ?;"""
        result = self.database.execute(sql, [user_id, ])
        if result:
            user = User(**result[0])
            return user
        return None

    async def get_user_by_telegram_id(self, telegram_id: int) -> (User or None):
        sql = """
                SELECT * 
                FROM users
                WHERE telegram_id = ?;"""
        result = self.database.execute(sql, [telegram_id, ])
        if result:
            user = User(**result[0])
            return user
        return None

    async def set_user_room_by_id(self, user_id: int, room: str) -> bool:
        sql = """
                UPDATE users
                SET room = ?
                WHERE id = ?;"""
        self.database.execute(sql, [room, user_id])
        self.database.commit()
        return True

    async def set_user_room_by_telegram_id(self, telegram_id: int, room: str) -> bool:
        sql = """
                UPDATE users
                SET room = ?
                WHERE telegram_id = ?;"""
        self.database.execute(sql, [room, telegram_id])
        self.database.commit()
        return True

    async def set_user_permission_by_telegram_id(self, telegram_id: int, permission: str) -> bool:
        sql = """
                   UPDATE users
                   SET permissions = ?
                   WHERE telegram_id = ?;"""
        self.database.execute(sql, [permission, telegram_id])
        self.database.commit()
        return True




from roombot.types.handler import Handler
from typing import Any, List


class Room:
    name: str
    kwargs: dict
    handler: Handler
    permissions: List[str or None]

    def __init__(self, room_name: str, room_handler: Handler, kwargs: dict, permissions: List[str or None]):
        self.name    = room_name
        self.kwargs  = kwargs
        self.handler = room_handler
        self.permissions = permissions


class RoomsStack:
    """
    Тут банально хранятся все данные о комнатах. Соритровка такая:
        1. Есть имя, оно как ключ в dict
        2. Этому имени соотвецтвует dict. В котором ключ это тип, а value - это комнаты.
    """
    rooms: dict[str, dict[int, List[Room]]]

    def __init__(self):
        self.rooms = {}

    def add(self, rooms: (Room or List[Room])):
        if isinstance(rooms, Room):
            rooms = [rooms]
        for room in rooms:
            if room.name not in self.rooms:
                self.rooms[room.name] = {}
            if not self.rooms[room.name].get(room.handler.handler_type):
                self.rooms[room.name][room.handler.handler_type] = []
            self.rooms[room.name][room.handler.handler_type].append(room)

    def get(self, room_name: str, get_by_type: int = None) -> List[Room]:
        rooms_by_name = self.rooms.get(room_name)
        if rooms_by_name is None:
            return []
        if get_by_type is not None:
            return rooms_by_name.get(get_by_type) or []
        return_list = []
        for i in rooms_by_name:
            return_list += rooms_by_name.get(i)
        return return_list



from typing import Any, List
from roombot.interfaces.IMassageHandler import IMessageHandler
from roombot.interfaces.ICallbackHandler import ICallbackHandler


class Handler:
    function: Any
    handler_type: int
    content_type: List[str]

    def __init__(self, h_type: int, content_type: List[str], func: Any):
        self.content_type = content_type
        self.handler_type = h_type
        self.function = func


class HandlersTypes:
    callback: int = 1
    message: int = 2
    on_join_message: int = 3
    on_join_callback: int = 4
    on_join_universal: int = 5
    compatibility = [
        [callback, on_join_callback, on_join_universal],
        [message, on_join_message, on_join_universal]
    ]

    @staticmethod
    def in_types(type_num: int):
        if type_num in (HandlersTypes.callback,
                        HandlersTypes.message,
                        HandlersTypes.on_join_message,
                        HandlersTypes.on_join_callback,
                        HandlersTypes.on_join_universal
                        ):
            return True
        return False

    @staticmethod
    def are_compatibility(*h_types):
        for i in HandlersTypes.compatibility:
            are_compat = True
            for h_type in h_types:
                if h_type not in i:
                    are_compat = False
            if are_compat:
                return True
        return False


class MessageHandlersStack:
    """
    Это все комнаты для обработки сообщений.
    """
    message_handlers: List[IMessageHandler]

    def __init__(self, *handlers):
        self.message_handlers = []
        self.message_handlers += handlers

    def add(self, handler: IMessageHandler):
        self.message_handlers.append(handler)
        return self

    def get_all(self):
        loaded_handlers = []
        for i in self.message_handlers:
            if not isinstance(i, IMessageHandler):
                continue
            loaded_handlers.append(i.process_message)
        return loaded_handlers


class CallbackHandlersStack:
    """
    Это все комнаты для обработки callback.
    """
    callback_handlers: List[ICallbackHandler]

    def __init__(self, *handlers):
        self.callback_handlers = []
        self.callback_handlers += handlers

    def add(self, handler: ICallbackHandler):
        self.callback_handlers.append(handler)
        return self

    def get_all(self):
        loaded_handlers = []
        for i in self.callback_handlers:
            if not isinstance(i, ICallbackHandler):
                continue
            loaded_handlers.append(i.process_callback)
        return loaded_handlers




from .types.dataclasses import User
from .types.handler import HandlersTypes
from .manager.manager import RoomsManager
from .default_databases.json_db import JsonDatabase
from .interfaces.IMassageHandler import IMessageHandler
from .default_databases.sqlite_db import Sqlite3Database
from .rooms_container.rooms_container import RoomsContainer
from .default_bots.aiogram_middleware import AiogramMiddleWare




import aiogram

from roombot.interfaces.ICallbackHandler import ICallbackHandler
from roombot.types.dataclasses import User
from roombot.types.handler import HandlersTypes
from roombot.interfaces.IUsersTable import IUsersTable
from roombot.types.room import Room
from roombot.utils.runfunc import run_func_as_async, get_kwargs_by_annotations, get_only_the_required_arguments


async def run_room_function(room: Room, callback: aiogram.types.CallbackQuery, internal_data, user: User):
    all_args = {aiogram.types.CallbackQuery: callback, User: user}
    all_args.update(internal_data.global_parameters)
    all_args.update(room.kwargs)
    args = get_kwargs_by_annotations(room.handler.function, all_args)
    args.update(get_only_the_required_arguments(room.handler.function, all_args))
    return await run_func_as_async(room.handler.function, **args)


class MainCallbackHandler(ICallbackHandler):
    def __init__(self, users: IUsersTable, rooms, internal_data, go_to_room):
        self.users = users
        self.rooms = rooms
        self.internal_data = internal_data
        self.go_to_room = go_to_room

    async def process_callback(self, callback: aiogram.types.CallbackQuery):
        user: User
        user_is_at_one_of_rooms: bool = False
        user = await self.users.get_user_by_telegram_id(callback.from_user.id)
        rooms = self.rooms.get(user.room, get_by_type=HandlersTypes.callback)
        if not rooms:
            return
        for room in rooms:
            user_is_at_one_of_rooms = True
            await run_room_function(room, callback, self.internal_data, user)
        if not user_is_at_one_of_rooms:
            await self.go_to_room(callback.from_user.id, self.internal_data.start_room)


class CheckUserCallbackHandler(ICallbackHandler):
    def __init__(self, users: IUsersTable, internal_data):
        self.users = users
        self.internal_data = internal_data

    async def process_callback(self, callback: aiogram.types.CallbackQuery):
        if not await self.users.get_user_by_telegram_id(callback.from_user.id):
            user_to_add = User(callback.from_user.id, callback.from_user.first_name, callback.from_user.last_name, self.internal_data.default_permissions, self.internal_data.start_room)
            await self.users.add_user(user_to_add)


class UserMigrationCallbackRoom(ICallbackHandler):
    def __init__(self, users: IUsersTable, rooms, internal_data, user_can_go_to_room):
        self.users = users
        self.rooms = rooms
        self.internal_data = internal_data
        self.user_can_go_to_room = user_can_go_to_room

    async def process_callback(self, callback: aiogram.types.CallbackQuery):
        user = await self.users.get_user_by_telegram_id(callback.from_user.id)
        user_migration = self.internal_data.users_migrations.get(user.telegram_id)
        if not user_migration: return
        on_join_callback_rooms = self.rooms.get(user_migration, get_by_type=HandlersTypes.on_join_callback) + self.rooms.get(user_migration, get_by_type=HandlersTypes.on_join_universal)
        if on_join_callback_rooms:
            for room in on_join_callback_rooms:
                if self.user_can_go_to_room(user, room):
                    await run_room_function(room, callback, self.internal_data, user)
                    await self.users.set_user_room_by_telegram_id(user.telegram_id, user_migration)
        else:
            for room in self.rooms.get(user_migration):
                if self.user_can_go_to_room(user, room):
                    await self.users.set_user_room_by_telegram_id(user.telegram_id, user_migration)
                    break

        if user.telegram_id in self.internal_data.users_migrations:
            self.internal_data.users_migrations.pop(user.telegram_id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from os import path

from setuptools import setup

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    wheel="",
    name='roombot',
    description="Mini framework for creating bots with tracking the user's position in the so-called \"rooms\". Based on aiogram.",
    version='0.0.20',
    url='https://github.com/drogi17/roombot',
    author='ic_it',
    author_email='',
    license='GNU 3',
    classifiers=[
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3'
    ],
    packages=[  'roombot', 
                "roombot.default_bots", 
                "roombot.default_databases", 
                "roombot.exception", 
                "roombot.interfaces", 
                "roombot.manager", 
                "roombot.rooms_container", 
                "roombot.types", 
                "roombot.utils"],
    install_requires=[
        'aiogram==2.11.2',
    ],
    long_description=long_description,
    long_description_content_type='text/markdown',
    python_requires=">=3.6",
    data_files=[("", ["LICENSE"]), ("", ["README.md"])]
)


import aiogram

from typing import List, Any
from dataclasses import dataclass
from .actions_with_user import ActionsWithUser
from roombot.types.room import RoomsStack
from roombot.types.dataclasses import User
from roombot.types.handler import MessageHandlersStack, CallbackHandlersStack
from roombot.interfaces.IUsersTable import IUsersTable
from roombot.rooms_container.rooms_container import RoomsContainer
from roombot.utils.runfunc import run_func_as_async, get_kwargs_by_annotations, get_only_the_required_arguments
from .default_message_handlers import MainMessageHandler, UserMigrationMessageRoom, CheckUserMessageHandler
from .default_callback_handlers import MainCallbackHandler, UserMigrationCallbackRoom, CheckUserCallbackHandler


@dataclass
class ManagerInternalData:
    """
    А это просто зборище всех очень нужных переменных, котрые в основном классе выглядели бы не крассиво и легче передвать именно
        dataclass, а не по одной переменной :)
    """
    start_room: str
    default_permissions: str
    users_migrations: dict
    global_parameters: dict


@dataclass
class HandlersHab:
    """
    Это короче то где будут хранится все ссылки на перехватчики событий.
    """
    message: MessageHandlersStack
    callback: CallbackHandlersStack


class RoomsManager:
    """
    Главный класс.
    Тут вроде все работает по человечески.
    """
    rooms: RoomsStack
    users: IUsersTable
    internal_data: ManagerInternalData
    handlers: HandlersHab
    default_handlers_included: bool

    def __init__(self,
                 users: IUsersTable,
                 start_room: str,
                 default_permissions: str = "",
                 include_default_handlers: bool = False
                 ):
        self.users = users
        self.rooms = RoomsStack()
        self.internal_data = ManagerInternalData(start_room, default_permissions, {}, {})
        self.handlers = HandlersHab(MessageHandlersStack(), CallbackHandlersStack())
        self.actions_with_user = ActionsWithUser(self.users, self.internal_data, self.rooms)
        self.default_handlers_included = False
        if include_default_handlers:
            self.include_default_handlers()

    def include_default_handlers(self):
        if self.default_handlers_included:
            return
        check_user_handler_message = CheckUserMessageHandler(self.users, self.internal_data)
        main_handler_message = MainMessageHandler(self.users, self.rooms, self.internal_data, self.actions_with_user.go_to_room)
        user_migration_handler_message = UserMigrationMessageRoom(self.users, self.rooms, self.internal_data,
                                                          self.actions_with_user.user_can_go_to_room)

        check_user_handler_callback = CheckUserCallbackHandler(self.users, self.internal_data)
        main_handler_callback = MainCallbackHandler(self.users, self.rooms, self.internal_data, self.actions_with_user.go_to_room)
        user_migration_handler_callback = UserMigrationCallbackRoom(self.users, self.rooms, self.internal_data,
                                                          self.actions_with_user.user_can_go_to_room)

        self.handlers.message.add(check_user_handler_message)
        self.handlers.message.add(main_handler_message)
        self.handlers.message.add(user_migration_handler_message)

        self.handlers.callback.add(check_user_handler_callback)
        self.handlers.callback.add(main_handler_callback)
        self.handlers.callback.add(user_migration_handler_callback)

        self.default_handlers_included = True

    def append_rooms(self, rooms_container: RoomsContainer):
        self.rooms.add(rooms_container.rooms)

    def add_message_handler(self, handler: Any):
        self.handlers.message.add(handler)

    def add_global_parameters(self, **kwargs):
        self.internal_data.global_parameters.update(kwargs)

    async def process_message(self, message: aiogram.types.Message):
        for handler in self.handlers.message.get_all():
            kwargs = get_kwargs_by_annotations(handler, {aiogram.types.Message: message
                                                         })
            await run_func_as_async(handler, **kwargs)

    async def process_callback(self, callback: aiogram.types.CallbackQuery):
        for handler in self.handlers.callback.get_all():
            all_args = {
                aiogram.types.CallbackQuery: callback
             }
            kwargs = get_kwargs_by_annotations(handler, all_args)
            await run_func_as_async(handler, **kwargs)

